import 'dart:convert';

import 'package:fis_i18n/i18n.dart';
import 'package:fis_measure/interfaces/process/workspace/application.dart';
import 'package:fis_measure/values/unit_desc.dart';
import 'package:fis_measure/view/paint/ai_patint_controller.dart';
import 'package:fis_measure/view/paint/date_structure.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vid/us/vid_us_unit.dart';

/// 特征分析
class FeatureAnalysis extends StatefulWidget {
  const FeatureAnalysis(this.descriptions, {Key? key}) : super(key: key);

  final List<AIDiagnosisDescription>? descriptions;

  @override
  State<StatefulWidget> createState() {
    return FeatureAnalysisState();
  }
}

class FeatureAnalysisState extends State<FeatureAnalysis> {
  late final aiPatintController = Get.find<AiPatintController>();
  double _unitsPhysicalPixels = 0;
  final application = Get.find<IApplication>();
  final ScrollController scrollController = ScrollController();
  String _xUnit = '';
  @override
  void initState() {
    _updateImagePhysicalSize();
    super.initState();
  }

  @override
  void didUpdateWidget(FeatureAnalysis oldWidget) {
    _updateImagePhysicalSize();

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    List<AIDiagnosisDescription> newDescriptions = [];
    for (var m in widget.descriptions!) {
      if (m.type != DiagnosisDescriptionEnum.CarotidIntimaMediaThickness &&
          m.type != DiagnosisDescriptionEnum.LesionSize) {
        newDescriptions.add(m);
      }
    }
    return Container(
      decoration: BoxDecoration(
        border: Border.all(
          color: const Color.fromRGBO(54, 169, 206, 1),
        ),
        borderRadius: BorderRadius.circular(4),
        color: Colors.transparent,
      ),
      child: Column(
        children: [
          Row(
            mainAxisSize: MainAxisSize.max,
            children: [
              Expanded(
                child: Container(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(4),
                      topRight: Radius.circular(4),
                    ),
                    color: Color.fromRGBO(54, 169, 206, 1),
                  ),
                  padding: const EdgeInsets.symmetric(
                    vertical: 4,
                    horizontal: 8,
                  ),
                  child: Text(
                    i18nBook.measure.featureAnalysis.t,
                    style: const TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ),
              )
            ],
          ),
          Scrollbar(
            isAlwaysShown: true,
            controller: scrollController,
            child: GridView.count(
              shrinkWrap: true,
              crossAxisCount: 2,
              childAspectRatio: 2,
              controller: scrollController,
              children: newDescriptions!.map((e) {
                return Container(
                  decoration: BoxDecoration(
                    border: Border.all(
                      color: const Color.fromRGBO(54, 169, 206, 1),
                    ),
                  ),
                  padding: EdgeInsets.symmetric(
                      horizontal: 4,
                      vertical: i18nBook.isCurrentRussian ? 2.5 : 4),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Tooltip(
                        message: _buildDescriptionType(e.type),
                        child: Text(
                          _buildDescriptionType(e.type),
                          style: const TextStyle(
                            color: Color.fromRGBO(54, 169, 206, 1),
                          ),
                          overflow: TextOverflow.ellipsis,
                          maxLines: 1,
                        ),
                      ),
                      Text(
                        _buildDescriptionValue(e.value ?? '', e.type),
                        style: const TextStyle(
                          color: Colors.white,
                          fontSize: 14,
                        ),
                      ),
                    ],
                  ),
                );
              }).toList(),
            ),
          ),
        ],
      ),
    );
  }

  String _buildDescriptionType(
      DiagnosisDescriptionEnum diagnosisDescriptionType) {
    switch (diagnosisDescriptionType) {
      case DiagnosisDescriptionEnum.Shape:
        return i18nBook.measure.shape.t;
      case DiagnosisDescriptionEnum.Orientation:
        return i18nBook.measure.orientation.t;
      case DiagnosisDescriptionEnum.EchoPattern:
        return i18nBook.measure.echoPattern.t;
      case DiagnosisDescriptionEnum.LesionBoundary:
        return i18nBook.measure.lesionBoundary.t;
      case DiagnosisDescriptionEnum.Margin:
        return i18nBook.measure.margin.t;
      case DiagnosisDescriptionEnum.Calcification:
        return i18nBook.measure.calcification.t;
      case DiagnosisDescriptionEnum.ThyroidEchoPattern:
        return i18nBook.measure.echoPattern.t;
      case DiagnosisDescriptionEnum.ThyroidShape:
        return i18nBook.measure.shape.t;
      case DiagnosisDescriptionEnum.ThyroidMargin:
        return i18nBook.measure.margin.t;
      case DiagnosisDescriptionEnum.ThyroidEchogenicFoci:
        return i18nBook.measure.thyroidEchogenicFoci.t;
      case DiagnosisDescriptionEnum.LiverShape:
        return i18nBook.measure.shape.t;
      case DiagnosisDescriptionEnum.LiverBoundary:
        return i18nBook.measure.lesionBoundary.t;
      case DiagnosisDescriptionEnum.LiverEchoTexture:
        return i18nBook.measure.liverEchoTexture.t;
      case DiagnosisDescriptionEnum.LesionSize:
        return i18nBook.measure.lesionSize.t;
      case DiagnosisDescriptionEnum.QlaqueEchoPattern: // 斑块回声类型
        return i18nBook.measure.plaqueEcho.t;
      case DiagnosisDescriptionEnum.QlaqueLocation: // 斑块位置
        return i18nBook.measure.plaquePosition.t;
      case DiagnosisDescriptionEnum.CarotidRateOfStenosis: // 颈动脉狭窄率
        return i18nBook.measure.stenosisRateoflumen.t;
      case DiagnosisDescriptionEnum.CarotidInnerDiameter: // 颈动脉内径
        return i18nBook.measure.carotidArteryDiameter.t;
      case DiagnosisDescriptionEnum.CarotidIntimaMediaThickness: // 颈动脉内中膜厚度
        return i18nBook.measure.innerMembraneThickness.t;
      default:
        print('diagnosisDescriptionType:$diagnosisDescriptionType');
        return '-';
    }
  }

  void _updateImagePhysicalSize() {
    if (application.visuals.isEmpty) {
      return;
    }
    if (application.visuals[0].visualAreas.isEmpty) {
      return;
    }
    VidUsUnit targetUnit =
        application.visuals[0].visualAreas[0].viewport?.xUnit ?? VidUsUnit.cm;
    var unitsPhysicalPixels =
        (application.visuals[0].visualAreas[0].viewport?.region.width)! /
            (application.frameData!.width).toDouble();
    setState(() {
      _xUnit = UnitDescriptionMap.getDesc(targetUnit);
      _unitsPhysicalPixels = unitsPhysicalPixels;
    });
  }

  String _buildDescriptionValue(
      String diagnosisDescriptionValue, DiagnosisDescriptionEnum type) {
    double? doubleValue = double.tryParse(diagnosisDescriptionValue);

    if (doubleValue != null) {
      var value = doubleValue.toStringAsFixed(2);
      if (type == DiagnosisDescriptionEnum.CarotidRateOfStenosis) {
        return value;
      }
      return '$value $_xUnit';
    }
    if (diagnosisDescriptionValue.length > 50) {
      final diagnosisDescription = jsonDecode(diagnosisDescriptionValue);
      double horizontalAndVerticalProportion =
          (diagnosisDescription?['VerticalLengthInPixel'] ?? 0) /
              (diagnosisDescription?['HorizontalLengthInPixel'] ?? 1);
      bool horizontalAndVertical = horizontalAndVerticalProportion > 1 ||
          horizontalAndVerticalProportion == 1;
      if (diagnosisDescription is Map<dynamic, dynamic> &&
          diagnosisDescription.containsKey('MeasuringLineLengthInPixel')) {
        double measuringLineLengthInPixel =
            diagnosisDescription['MeasuringLineLengthInPixel'] ?? 0;

        if (measuringLineLengthInPixel <= 0) {
          return '< 1';
        }
        var result = (measuringLineLengthInPixel * _unitsPhysicalPixels)
            .toStringAsFixed(2)
            .toString();
        return '$result $_xUnit';
      }
      return horizontalAndVertical ? '> 1' : '< 1';
    } else {
      switch (diagnosisDescriptionValue) {
        case 'Oval':
          return i18nBook.measure.oval.t;
        case 'Round':
          return i18nBook.measure.round.t;
        case 'Irregular':
          return i18nBook.measure.irregular.t;
        case 'Regular':
          return i18nBook.measure.regular.t;
        case 'Homogeneous':
          return i18nBook.measure.homogeneous.t;
        case 'Heterogeneous':
          return i18nBook.measure.heterogeneous.t;
        case 'Parallel':
          return i18nBook.measure.parallel.t;
        case 'NonParallel':
          return i18nBook.measure.nonParallel.t;
        case 'Anechoic':
          return i18nBook.measure.anechoic.t;
        case 'Hypoechoic':
          return i18nBook.measure.hypoechoic.t;
        case 'Isoechoic':
          return i18nBook.measure.isoechoic.t;
        case 'Hyperechoic':
          return i18nBook.measure.hyperechoic.t;
        case 'Complex':
          return i18nBook.measure.complex.t;
        case 'Strongechoic':
          return i18nBook.measure.strongechoic.t;
        case 'AbruptInterface':
          return i18nBook.measure.abruptInterface.t;
        case 'EchogenicHalo':
          return i18nBook.measure.echogenicHalo.t;
        case 'Circumscribed':
          return i18nBook.measure.circumscribed.t;
        case 'NonCircumscribed':
          return i18nBook.measure.nonCircumscribed.t;
        case 'NoCalcifications':
          return i18nBook.measure.noCalcifications.t;
        case 'Macrocalcifications':
          return i18nBook.measure.macrocalcifications.t;
        case 'CalcificationsInMass':
          return i18nBook.measure.calcificationsInMass.t;
        case 'CalcificationsOutOfMass':
          return i18nBook.measure.calcificationsOutOfMass.t;
        case 'WiderThanTall':
          return i18nBook.measure.widerThanTall.t;
        case 'TallThanWider':
          return i18nBook.measure.tallThanWider.t;
        case 'Smooth':
          return i18nBook.measure.smooth.t;
        case 'IllDefined':
          return i18nBook.measure.illDefined.t;
        case 'Lobulated':
          return i18nBook.measure.lobulated.t;
        case 'ExtraThyroidalExtension':
          return i18nBook.measure.extraThyroidalExtension.t;
        case 'NoCifications':
          return i18nBook.measure.noCifications.t;
        case 'Coarsecalcifications':
          return i18nBook.measure.coarsecalcifications.t;
        case 'Microcalcifications':
          return i18nBook.measure.microcalcifications.t;
        case 'AnteriorWall':
          return i18nBook.measure.frontWall.t;
        case 'PosteriorWall':
          return i18nBook.measure.rearWall.t;
        case 'SideWall':
          return i18nBook.measure.sideWall.t;
        default:
          return '';
      }
    }
  }
}
