import 'package:fis_measure/interfaces/process/player/play_controller.dart';
import 'package:fis_measure/view/player/controller.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class CrossFrameAnchorBar extends StatefulWidget {
  const CrossFrameAnchorBar({super.key});

  @override
  State<StatefulWidget> createState() => _CrossFrameAnchorBarState();
}

class _CrossFrameAnchorBarState extends State<CrossFrameAnchorBar> {
  final controller = Get.find<IPlayerController>() as VidPlayerController;

  late List<int> anchorList;
  late int totalCount;

  @override
  void initState() {
    anchorList = controller.crossFrameAnchors;
    totalCount = controller.totalFramesCount;
    controller.crossFrameAnchorsUpdated.addListener(_onAnchorsUpdated);
    super.initState();
  }

  @override
  void dispose() {
    controller.crossFrameAnchorsUpdated.removeListener(_onAnchorsUpdated);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (context, c) {
        return RepaintBoundary(
          child: CustomPaint(
            painter: _AnchorBarPainter(totalCount, anchorList),
            size: Size(c.maxWidth, c.maxHeight),
          ),
        );
      },
    );
  }

  void _onAnchorsUpdated(Object _, List<int> e) {
    setState(() {
      anchorList = e;
    });
  }
}

class _AnchorBarPainter extends CustomPainter {
  static double thumbRadius = 10.0;
  final int totalCount;
  final List<int> anchors;

  _AnchorBarPainter(this.totalCount, this.anchors);

  final painter = Paint()
    ..strokeWidth = 2
    ..color = Colors.red;

  double unitWidth = 0.0;
  double height = 0.0;
  double centerY = 0.0;

  @override
  void paint(Canvas canvas, Size size) {
    height = size.height;
    centerY = height / 2;
    // unitWidth = size.width / totalCount;
    unitWidth = (size.width + thumbRadius * 2) / totalCount;
    for (var anchor in anchors) {
      drawOne(canvas, anchor);
    }
  }

  void drawOne(Canvas canvas, int anchor) {
    // Offset offset;
    // if (anchor == 0) {
    // offset = Offset(unitWidth * anchor, 0);
    // } else {
    //   offset = Offset(unitWidth * anchor - (thumbRadius / 2), 0);
    // }
    // final endOffset = offset.translate(0, height);

    final x = unitWidth * anchor;
    final offset = Offset(x, centerY - thumbRadius - 4);
    final endOffset = Offset(x, centerY + 4);
    canvas.drawLine(offset, endOffset, painter);
  }

  @override
  bool shouldRepaint(covariant _AnchorBarPainter oldDelegate) {
    if (oldDelegate.anchors != anchors) {
      return true;
    }
    if (oldDelegate.totalCount != totalCount) {
      return true;
    }
    return false;
  }
}
